@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 #
-# Copyright (C) 2024 Advanced Micro Devices. All rights reserved.
+# Copyright (C) Advanced Micro Devices. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy of
 # this software and associated documentation files (the "Software"), to deal in
@@ -18,20 +18,20 @@
 # COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 # IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
 import logging
 import math
 import os
+import grp
 import platform
 import sys
 import time
 import re
 import multiprocessing
+import json
 
-from typing import List, Union
 from enum import Enum
-from typing import Set
+from typing import List, Set, Union
 
 from amdsmi_init import *
 from BDF import BDF
@@ -57,6 +57,7 @@ class AMDSMIHelpers():
         self._is_windows = False
         self._count_of_sets_called = 0
 
+        # Check if the system is a virtual OS
         if self.operating_system.startswith("Linux"):
             self._is_linux = True
             logging.debug(f"AMDSMIHelpers: Platform is linux:{self._is_linux}")
@@ -70,21 +71,38 @@ class AMDSMIHelpers():
 
             self._is_baremetal = not self._is_virtual_os
 
-            # Check for passthrough system filtering by device id
+        if self._is_virtual_os:
+            #If hard coded passthrough device ids exist on Virtual OS,
+            #   then it is a passthrough system
             output = self.get_pci_device_ids()
             passthrough_device_ids = ["7460", "73c8", "74a0", "74a1", "74a2"]
             if any(('0x' + device_id) in output for device_id in passthrough_device_ids):
-                if self._is_virtual_os:
-                    self._is_baremetal = True
-                    self._is_virtual_os = False
-                    self._is_passthrough = True
+                self._is_baremetal = True
+                self._is_virtual_os = False
+                self._is_passthrough = True
+
+            # Check for passthrough system dynamically via drm querying id_flags
+            try:
+                if self.is_amdgpu_initialized() and not self._is_passthrough:
+                    device_handles = amdsmi_interface.amdsmi_get_processor_handles()
+                    for dev in device_handles:
+                        virtualization_info = amdsmi_interface.amdsmi_get_gpu_virtualization_mode(dev)
+                        if virtualization_info['mode'] == amdsmi_interface.AmdSmiVirtualizationMode.PASSTHROUGH:
+                            self._is_baremetal = True
+                            self._is_virtual_os = False
+                            self._is_passthrough = True
+                            break # Once passthrough is determined, we can immediately break
+            except amdsmi_exception.AmdSmiLibraryException as e:
+                logging.debug("Unable to determine virtualization status: " + str(e.get_error_code()))
 
     def increment_set_count(self):
         self._count_of_sets_called += 1
 
+
     def get_set_count(self):
         return self._count_of_sets_called
 
+
     def is_virtual_os(self):
         return self._is_virtual_os
 
@@ -157,6 +175,16 @@ class AMDSMIHelpers():
         return AMDSMI_INIT_FLAG & amdsmi_interface.amdsmi_wrapper.AMDSMI_INIT_AMD_CPUS
 
 
+    def get_rocm_version(self):
+        try:
+            rocm_lib_status, rocm_version = amdsmi_interface.amdsmi_get_rocm_version()
+            if rocm_lib_status is not True:
+                return "N/A"
+            return rocm_version
+        except amdsmi_interface.AmdSmiLibraryException as e:
+            return "N/A"
+
+
     def get_cpu_choices(self):
         """Return dictionary of possible CPU choices and string of the output:
             Dictionary will be in format: cpus[ID]: Device Handle)
@@ -682,12 +710,30 @@ class AMDSMIHelpers():
         perf_levels_int = list(set(clock.value for clock in amdsmi_interface.AmdSmiDevPerfLevel))
         return perf_levels_str, perf_levels_int
 
+    def get_accelerator_partition_profile_config(self):
+        device_handles = amdsmi_interface.amdsmi_get_processor_handles()
+        accelerator_partition_profiles = {'profile_indices':[], 'profile_types':[], 'memory_caps': []}
+        for dev in device_handles:
+            try:
+                profile = amdsmi_interface.amdsmi_get_gpu_accelerator_partition_profile_config(dev)
+                num_profiles = profile['num_profiles']
+                for p in range(num_profiles):
+                    accelerator_partition_profiles['profile_indices'].append(str(profile['profiles'][p]['profile_index']))
+                    accelerator_partition_profiles['profile_types'].append(profile['profiles'][p]['profile_type'])
+                    accelerator_partition_profiles['memory_caps'].append(profile['profiles'][p]['memory_caps'])
+                break # Only need to get the profiles for one device
+            except amdsmi_interface.AmdSmiLibraryException as e:
+                break
+        return accelerator_partition_profiles
 
-    def get_compute_partition_types(self):
-        compute_partitions_str = [partition.name for partition in amdsmi_interface.AmdSmiComputePartitionType]
-        if 'INVALID' in compute_partitions_str:
-            compute_partitions_str.remove('INVALID')
-        return compute_partitions_str
+    def get_accelerator_choices_types_indices(self):
+        return_val = ("N/A", {'profile_indices':[], 'profile_types':[]})
+        accelerator_partition_profiles = self.get_accelerator_partition_profile_config()
+        if len(accelerator_partition_profiles['profile_types']) != 0:
+            compute_partitions_str = accelerator_partition_profiles['profile_types'] + accelerator_partition_profiles['profile_indices']
+            accelerator_choices = ", ".join(compute_partitions_str)
+            return_val = (accelerator_choices, accelerator_partition_profiles)
+        return return_val
 
     def get_memory_partition_types(self):
         memory_partitions_str = [partition.name for partition in amdsmi_interface.AmdSmiMemoryPartitionType]
@@ -702,6 +748,67 @@ class AMDSMIHelpers():
         return clock_types_str, clock_types_int
 
 
+    def get_power_profiles(self):
+        power_profiles_str = [profile.name for profile in amdsmi_interface.AmdSmiPowerProfilePresetMasks]
+        if 'UNKNOWN' in power_profiles_str:
+            power_profiles_str.remove('UNKNOWN')
+        return power_profiles_str
+
+
+    def get_perf_det_levels(self):
+        perf_det_level_str = [level.name for level in amdsmi_interface.AmdSmiDevPerfLevel]
+        if 'UNKNOWN' in perf_det_level_str:
+            perf_det_level_str.remove('UNKNOWN')
+        return perf_det_level_str
+
+
+    def get_power_caps(self):
+        device_handles = amdsmi_interface.amdsmi_get_processor_handles()
+        power_cap_min = amdsmi_interface.MaxUIntegerTypes.UINT64_T # start out at max and min and then find real min and max
+        power_cap_max = 0
+        for dev in device_handles:
+            power_cap_info = amdsmi_interface.amdsmi_get_power_cap_info(dev)
+            if power_cap_info['max_power_cap'] > power_cap_max:
+                power_cap_max = power_cap_info['max_power_cap']
+            if power_cap_info['min_power_cap'] < power_cap_max:
+                power_cap_min = power_cap_info['min_power_cap']
+        return (power_cap_min, power_cap_max)
+
+
+    def get_soc_pstates(self):
+        device_handles = amdsmi_interface.amdsmi_get_processor_handles()
+        soc_pstate_profile_list = []
+        for dev in device_handles:
+            try:
+                soc_pstate_info = amdsmi_interface.amdsmi_get_soc_pstate(dev)
+            except amdsmi_interface.AmdSmiLibraryException as e:
+                continue
+            for policy in soc_pstate_info['policies']:
+                policy_string = f"{policy['policy_id']}: {policy['policy_description']}"
+                if not policy_string in soc_pstate_profile_list:
+                    soc_pstate_profile_list.append(policy_string)
+        if len(soc_pstate_profile_list) == 0:
+            soc_pstate_profile_list.append("N/A")
+        return soc_pstate_profile_list
+
+
+    def get_xgmi_plpd_policies(self):
+        device_handles = amdsmi_interface.amdsmi_get_processor_handles()
+        xgmi_plpd_profile_list = []
+        for dev in device_handles:
+            try:
+                xgmi_plpd_info = amdsmi_interface.amdsmi_get_xgmi_plpd(dev)
+            except amdsmi_interface.AmdSmiLibraryException as e:
+                continue
+            for policy in xgmi_plpd_info['plpds']:
+                policy_string = f"{policy['policy_id']}: {policy['policy_description']}"
+                if not policy_string in xgmi_plpd_profile_list:
+                    xgmi_plpd_profile_list.append(policy_string)
+        if len(xgmi_plpd_profile_list) == 0:
+            xgmi_plpd_profile_list.append("N/A")
+        return xgmi_plpd_profile_list
+
+
     def validate_clock_type(self, input_clock_type):
         valid_clock_types_str, valid_clock_types_int = self.get_clock_types()
 
@@ -781,6 +888,8 @@ class AMDSMIHelpers():
 
 
     def convert_bytes_to_readable(self, bytes_input, format_length=None):
+        if isinstance(bytes_input, str):
+            return "N/A"
         for unit in ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB"]:
             if abs(bytes_input) < 1024:
                 if format_length is not None:
@@ -860,11 +969,15 @@ class AMDSMIHelpers():
                 continue
         return pci_devices
 
-    def progressbar(self, it, prefix="", size=60, out=sys.stdout):
+    def progressbar(self, it, prefix="", size=60, out=sys.stdout, add_newline=False):
         count = len(it)
+        if (add_newline):
+            print("{}\n".format(prefix),end='\r', file=out, flush=False)
+        else:
+            print("{}".format(prefix),end='\r', file=out, flush=False)
         def show(j):
             x = int(size*j/count)
-            print("{}[{}{}] {}/{} secs remain".format(prefix, u"█"*x, "."*(size-x), j, count),
+            print("[{}{}] {}/{} secs remain".format(u"█"*x, "."*(size-x), j, count),
                     end='\r', file=out, flush=True)
         show(0)
         for i, item in enumerate(it):
@@ -872,8 +985,34 @@ class AMDSMIHelpers():
             show(i+1)
         print("\n\n", end='\r', flush=True, file=out)
 
-    def showProgressbar(self, title="", timeInSeconds=13):
+    def showProgressbar(self, title="", timeInSeconds=13, add_newline=False):
         if title != "":
-            title += ": "
-        for i in self.progressbar(range(timeInSeconds), title, 40):
+            title += " "
+        for i in self.progressbar(range(timeInSeconds), title, 40, add_newline=add_newline):
             time.sleep(1)
+
+    def check_required_groups(self):
+        """
+        Check if the current user is a member of the required groups.
+        If not, log a warning.
+        """
+
+        # Skip check if running as root.
+        if os.geteuid() == 0:
+            return
+
+        required_groups = {'video', 'render'}
+        try:
+            user_groups = {grp.getgrgid(gid).gr_name for gid in os.getgroups()}
+        except Exception as e:
+            logging.warning("Unable to determine group memberships: %s", e)
+            return
+
+        missing_groups = required_groups - user_groups
+        if missing_groups:
+            msg = (
+                "WARNING: User is missing the following required groups: %s. "
+                "Please add user to these groups."
+            ) % ", ".join(sorted(missing_groups))
+            print(msg)
+            logging.warning(msg)
