@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 #
-# Copyright (C) 2024 Advanced Micro Devices. All rights reserved.
+# Copyright (C) Advanced Micro Devices. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy of
 # this software and associated documentation files (the "Software"), to deal in
@@ -18,7 +18,6 @@
 # COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 # IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
 import csv
 import json
@@ -26,20 +25,10 @@ import re
 import time
 from typing import Dict
 from enum import Enum
-import yaml
 import inspect
-
 from amdsmi_helpers import AMDSMIHelpers
 import amdsmi_cli_exceptions
 
-### Custom YAML Functions
-# Dumper class to preserve order of yaml.dump
-class CustomDumper(yaml.Dumper):
-    def represent_dict_preserve_order(self, data):
-        return self.represent_dict(data.items())
-def has_sort_keys_option(): # to check if sort_keys is available
-    return 'sort_keys' in inspect.signature(yaml.dump).parameters
-
 class AMDSMILogger():
     def __init__(self, format='human_readable', destination='stdout') -> None:
         self.output = {}
@@ -51,6 +40,7 @@ class AMDSMILogger():
         self.table_header = ""
         self.secondary_table_title = ""
         self.secondary_table_header = ""
+        self.warning_message = ""
         self.helpers = AMDSMIHelpers()
 
 
@@ -84,7 +74,7 @@ class AMDSMILogger():
         return self.format == self.LoggerFormat.human_readable.value
 
 
-    def clear_multiple_devices_ouput(self):
+    def clear_multiple_devices_output(self):
         self.multiple_device_output.clear()
 
 
@@ -113,14 +103,24 @@ class AMDSMILogger():
         return output_dict
 
 
-    def _convert_json_to_tabular(self, json_object: Dict[str, any]):
-        # TODO make dynamic
+    def _convert_json_to_tabular(self, json_object: Dict[str, any], dynamic=False):
+        # TODO make dynamic - convert other python CLI outputs to use (as needed)
+        # Update: using dynamic=true provides dynamic re-sizing based on key name length
+
         table_values = ''
         stored_gpu = ''
         stored_timestamp = ''
         for key, value in json_object.items():
             string_value = str(value)
-            if key == 'gpu':
+            if key == 'partition_id':
+                # Special case for partition_id: 8 partitions + 7 comma + 2 spaces = 17
+                table_values += string_value.ljust(17)
+                continue
+            key_length = len(key) + 2
+            if dynamic and len(key) > 0:
+                stored_gpu = string_value
+                table_values += string_value.ljust(key_length)
+            elif key == 'gpu':
                 stored_gpu = string_value
                 table_values += string_value.rjust(3)
             elif key == 'timestamp':
@@ -128,13 +128,21 @@ class AMDSMILogger():
                 table_values += string_value.rjust(10) + '  '
             elif key == 'power_usage':
                 table_values += string_value.rjust(7)
-            elif key in ('gfx_clock', 'mem_clock', 'vram_used'):
-                table_values += string_value.rjust(11)
-            elif key in ('vclock', 'dclock'):
+            elif key in ('hotspot_temperature', 'memory_temperature'):
                 table_values += string_value.rjust(8)
-            elif key == 'vram_total' or 'ecc' in key or key == 'pcie_bw':
+            elif key in ('gfx', 'mem'):
+                table_values += string_value.rjust(7)
+            elif key in ('gfx_clk'):
+                table_values += string_value.rjust(10)
+            elif key in ('mem_clock', 'vram_used'):
+                table_values += string_value.rjust(11)
+            elif key in ('encoder', 'decoder'):
+                table_values += string_value.rjust(7)
+            elif key in ('vclock', 'dclock'):
+                table_values += string_value.rjust(10)
+            elif key in ('single_bit_ecc', 'double_bit_ecc', 'pcie_bw', 'vram_total'):
                 table_values += string_value.rjust(12)
-            elif key in ['pcie_replay']:
+            elif key in ('pcie_replay'):
                 table_values += string_value.rjust(13)
             # Only for handling topology tables
             elif 'gpu_' in key:
@@ -143,41 +151,32 @@ class AMDSMILogger():
             elif key == "gpu#":
                 table_values += string_value.ljust(7)
             elif key == "bdf":
-                table_values += string_value.ljust(13)
+                table_values += string_value.ljust(14)
             elif "bdf_" in key:
                 table_values += string_value.ljust(13)
             elif key == "bit_rate":
-                table_values += string_value.ljust(9)
-            elif key == "max_bandwidth":
-                table_values += string_value.ljust(14)
-            elif key == "link_type":
                 table_values += string_value.ljust(10)
-            elif key == "memory":
-                table_values += string_value.ljust(8)
-            elif key == "accelerator_type":
-                table_values += string_value.ljust(18)
-            elif key == "partition_id":
-                table_values += string_value.ljust(14)
-            elif key == "accelerator_profile_index":
-                table_values += string_value.ljust(27)
-            elif key == "profile_index":
+            elif key == "max_bandwidth":
                 table_values += string_value.ljust(15)
-            elif key == "memory_partition_caps":
-                table_values += string_value.ljust(23)
-            elif key == "num_partitions":
-                table_values += string_value.ljust(16)
-            elif key == "num_resources":
-                table_values += string_value.ljust(15)
-            elif key == "resource_index":
-                table_values += string_value.ljust(16)
-            elif key == "resource_type":
-                table_values += string_value.ljust(15)
-            elif key == "resource_instances":
-                table_values += string_value.ljust(20)
-            elif key == "resources_shared":
-                table_values += string_value.ljust(18)
+            elif key == "link_type":
+                table_values += string_value.ljust(11)
+            elif key == "link_status":
+                for i in value:
+                    table_values += str(i).ljust(3)
             elif key == "RW":
-                table_values += string_value.ljust(53)
+                table_values += string_value.ljust(57)
+            elif key in ('pviol', 'tviol'):
+                table_values += string_value.rjust(7)
+            elif key == "tviol_active":
+                table_values += string_value.rjust(14)
+            elif key == "phot_tviol":
+                table_values += string_value.rjust(12)
+            elif key == "vr_tviol":
+                table_values += string_value.rjust(10)
+            elif key == "hbm_tviol":
+                table_values += string_value.rjust(11)
+            elif key == "gfx_clkviol":
+                table_values += string_value.rjust(13)
             elif key == "process_list":
                 #Add an additional padding between the first instance of GPU and NAME
                 table_values += '  '
@@ -236,15 +235,8 @@ class AMDSMILogger():
 
         capitalized_json["AMDSMI_SPACING_REMOVAL"] = tabbed_dictionary
 
-        json_string = json.dumps(capitalized_json, indent=4)
-
-        if has_sort_keys_option():
-            yaml_data = yaml.safe_load(json_string)
-            yaml_output = yaml.dump(yaml_data, sort_keys=False, allow_unicode=True)
-        else:
-            CustomDumper.add_representer(dict, CustomDumper.represent_dict_preserve_order)
-            yaml_data = yaml.safe_load(json_string)
-            yaml_output = yaml.dump(yaml_data, Dumper=CustomDumper, allow_unicode=True, default_flow_style=False)
+        # Convert the capitalized JSON to a YAML-like string
+        yaml_output = self.custom_dump(capitalized_json)
 
         # Remove a key line if it is a spacer
         yaml_output = yaml_output.replace("AMDSMI_SPACING_REMOVAL:\n", "")
@@ -267,6 +259,22 @@ class AMDSMILogger():
 
         return clean_yaml_output
 
+    def custom_dump(self, data, indent=0):
+        """Converts a Python dictionary to a YAML-like string."""
+        yaml_string = ""
+        for key, value in data.items():
+            if isinstance(value, dict):
+                yaml_string += "  " * indent + f"{key}:\n" + self.custom_dump(value, indent + 1)
+            elif isinstance(value, list):
+                yaml_string += "  " * indent + f"{key}:\n"
+                for item in value:
+                    if isinstance(item, dict):
+                        yaml_string += self.custom_dump(item, indent + 1)
+                    else: # If the list is not a dictionary, print it as a string
+                        yaml_string += "  " * (indent + 1) + f"- {item}\n"
+            else:
+                yaml_string += "  " * indent + f"{key}: {value}\n"
+        return yaml_string
 
     def flatten_dict(self, target_dict, topology_override=False):
         """This will flatten a dictionary out to a single level of key value stores
@@ -481,12 +489,14 @@ class AMDSMILogger():
             self.output = {}
 
 
-    def print_output(self, multiple_device_enabled=False, watching_output=False, tabular=False, dual_csv_output=False):
+    def print_output(self, multiple_device_enabled=False, watching_output=False, tabular=False, dual_csv_output=False, dynamic=False):
         """ Print current output acording to format and then destination
             params:
                 multiple_device_enabled (bool) - True if printing output from
                     multiple devices
                 watching_output (bool) - True if printing watch output
+                dynamic (bool) - Defaults to False. True turns on dynamic resizing for
+                    left justified table output
             return:
                 Nothing
         """
@@ -503,7 +513,7 @@ class AMDSMILogger():
         elif self.is_human_readable_format():
             # If tabular output is enabled, redirect to _print_tabular_output
             if tabular:
-                self._print_tabular_output(multiple_device_enabled=multiple_device_enabled, watching_output=watching_output)
+                self._print_tabular_output(multiple_device_enabled=multiple_device_enabled, watching_output=watching_output, dynamic=dynamic)
             else:
                 self._print_human_readable_output(multiple_device_enabled=multiple_device_enabled,
                                                    watching_output=watching_output)
@@ -775,7 +785,7 @@ class AMDSMILogger():
                     output_file.write(human_readable_output + '\n')
 
 
-    def _print_tabular_output(self, multiple_device_enabled=False, watching_output=False):
+    def _print_tabular_output(self, multiple_device_enabled=False, watching_output=False, dynamic=False):
         primary_table = ''
         secondary_table = ''
 
@@ -795,7 +805,7 @@ class AMDSMILogger():
                 for key, value in device_output.items():
                     if key != 'process_list':
                         primary_table_output[key] = value
-                primary_table += self._convert_json_to_tabular(primary_table_output) + '\n'
+                primary_table += self._convert_json_to_tabular(primary_table_output, dynamic=dynamic) + '\n'
         else: # Single device output
             if 'process_list' in self.output:
                 process_table_dict = {}
@@ -809,7 +819,7 @@ class AMDSMILogger():
             for key, value in self.output.items():
                 if key != 'process_list':
                     primary_table_output[key] = value
-            primary_table += self._convert_json_to_tabular(primary_table_output) + '\n'
+            primary_table += self._convert_json_to_tabular(primary_table_output, dynamic=dynamic) + '\n'
         primary_table = primary_table.rstrip()
         secondary_table = secondary_table.rstrip()
 
@@ -818,6 +828,8 @@ class AMDSMILogger():
             primary_table_heading = ''
             if self.table_title:
                 primary_table_heading = self.table_title + ':\n'
+            if self.warning_message:  # Add warning message below the table title
+                primary_table_heading += self.warning_message + '\n'
             primary_table_heading += self.table_header + '\n'
             primary_table = primary_table_heading + primary_table
 
@@ -866,7 +878,7 @@ class AMDSMILogger():
                         for key, value in device_output.items():
                             if key != 'process_list':
                                 primary_table_output[key] = value
-                        primary_table += self._convert_json_to_tabular(primary_table_output) + '\n'
+                        primary_table += self._convert_json_to_tabular(primary_table_output, dynamic=dynamic) + '\n'
                     primary_table = primary_table.rstrip() # Remove trailing new line
                     secondary_table = secondary_table.rstrip()
 
@@ -875,6 +887,8 @@ class AMDSMILogger():
                         primary_table_heading = ''
                         if self.table_title:
                             primary_table_heading = self.table_title + ':\n'
+                        if self.warning_message: # Add warning message below the table title
+                            primary_table_heading += self.warning_message + '\n'
                         primary_table_heading += self.table_header + '\n'
                         primary_table = primary_table_heading + primary_table
 
