@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 #
-# Copyright (C) 2024 Advanced Micro Devices. All rights reserved.
+# Copyright (C) Advanced Micro Devices. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy of
 # this software and associated documentation files (the "Software"), to deal in
@@ -18,8 +18,6 @@
 # COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 # IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
-
 
 import argparse
 import errno
@@ -27,6 +25,7 @@ import os
 import sys
 import time
 import collections
+from amdsmi import amdsmi_interface
 from typing import Optional
 from typing import Union
 
@@ -34,7 +33,6 @@ from pathlib import Path
 
 from _version import __version__
 from amdsmi_helpers import AMDSMIHelpers
-from rocm_version import get_rocm_version
 import amdsmi_cli_exceptions
 
 
@@ -96,7 +94,7 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         version_string = f"Version: {__version__}"
         platform_string = f"Platform: {self.helpers.os_info()}"
-        rocm_version = get_rocm_version()
+        rocm_version = self.helpers.get_rocm_version()
         rocm_version_string = f"ROCm version: {rocm_version}"
         program_name = 'amd-smi'
 
@@ -138,19 +136,18 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_partition_parser(self.subparsers, partition)
 
 
-    def _not_negative_int(self, int_value):
+    def _not_negative_int(self, int_value, sub_arg=None):
         # Argument type validator
         if int_value.isdigit():  # Is digit doesn't work on negative numbers
             return int(int_value)
 
         outputformat = self.helpers.get_output_format()
         if int_value == "":
-            raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException(int_value, outputformat)
+            raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException(sub_arg, outputformat)
         else:
             raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(int_value, outputformat)
 
-
-    def _positive_int(self, int_value):
+    def _positive_int(self, int_value, sub_arg=None):
         # Argument type validator
         if int_value.isdigit():  # Is digit doesn't work on negative numbers
             if int(int_value) > 0:
@@ -158,12 +155,12 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         outputformat = self.helpers.get_output_format()
         if int_value == "":
-            raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException(int_value, outputformat)
+            raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException(sub_arg, outputformat)
         else:
             raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(int_value, outputformat)
 
 
-    def _is_valid_string(self, string_value):
+    def _is_valid_string(self, string_value, sub_arg=None):
         # Argument type validator
         # This is for triggering a cli exception if an empty string is detected
         if string_value:
@@ -171,10 +168,18 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         outputformat = self.helpers.get_output_format()
         if string_value == "":
-            raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException(string_value, outputformat)
+            raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException(sub_arg, outputformat)
         else:
             raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(string_value, outputformat)
 
+    def _is_command_supported(self, user_input, acceptable_values, command_name):
+        if acceptable_values == "N/A":
+            raise amdsmi_cli_exceptions.AmdSmiCommandNotSupportedException(command_name, self.helpers.get_output_format())
+        elif str(user_input).upper() not in acceptable_values:
+            print(f"Valid inputs are {acceptable_values}")
+            raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(str(user_input).upper(), self.helpers.get_output_format())
+        else:
+            return str(user_input).upper()
 
     def _limit_select(self):
         """Custom action for setting clock limits"""
@@ -205,6 +210,37 @@ class AMDSMIParser(argparse.ArgumentParser):
         return AMDSMILimitArgs
 
 
+    def _level_select(self):
+        """Custom action for setting clock frequencies to particular performance level"""
+        output_format = self.helpers.get_output_format()
+
+        class AMDSMIFreqArgs(argparse.Action):
+            def __call__(self, parser: AMDSMIParser, namespace: argparse.Namespace,
+                         values: list, option_string: Optional[str] = None) -> None:
+                # valid values
+                valid_clk_types = ('sclk', 'mclk', 'pcie', 'fclk', 'socclk')
+                clk_type = values[0]
+                perf_levels_str = values[1:]
+
+                # Check if the sclk and mclk parameters are valid
+                if clk_type not in valid_clk_types:
+                    raise amdsmi_cli_exceptions.AmdSmiInvalidParameterException(clk_type, output_format)
+
+                perf_levels = []
+                # Check if every item in perf level is valid
+                for level in perf_levels_str:
+                    if not level.isdigit():
+                        raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(level, output_format)
+                    level = int(level)
+                    if level < 0:
+                        raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(level, output_format)
+                    perf_levels.append(level)
+
+                clk_level_args = collections.namedtuple('clk_level_args', ['clk_type', 'perf_levels'])
+                setattr(namespace, self.dest, clk_level_args(clk_type, perf_levels))
+        return AMDSMIFreqArgs
+
+
     def _check_output_file_path(self):
         """ Argument action validator:
             Returns a path to a file from the output file path provided.
@@ -291,7 +327,7 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         amdsmi_helpers = self.helpers
         class _GPUSelectAction(argparse.Action):
-            ouputformat=self.helpers.get_output_format()
+            outputformat=self.helpers.get_output_format()
             # Checks the values
             def __call__(self, parser, args, values, option_string=None):
                 if "all" in gpu_choices:
@@ -302,13 +338,13 @@ class AMDSMIParser(argparse.ArgumentParser):
                     setattr(args, self.dest, selected_device_handles)
                 else:
                     if selected_device_handles == '':
-                        raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException("--gpu", _GPUSelectAction.ouputformat)
+                        raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException("--gpu", _GPUSelectAction.outputformat)
                     elif not gpu_format:
                         raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(selected_device_handles,
-                                                                                         _GPUSelectAction.ouputformat)
+                                                                                         _GPUSelectAction.outputformat)
                     else:
                         raise amdsmi_cli_exceptions.AmdSmiDeviceNotFoundException(selected_device_handles,
-                                                                                  _GPUSelectAction.ouputformat,
+                                                                                  _GPUSelectAction.outputformat,
                                                                                   True, False, False)
 
         return _GPUSelectAction
@@ -321,7 +357,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         """
         amdsmi_helpers = self.helpers
         class _CPUSelectAction(argparse.Action):
-            ouputformat=self.helpers.get_output_format()
+            outputformat=self.helpers.get_output_format()
             # Checks the values
             def __call__(self, parser, args, values, option_string=None):
                 if "all" in cpu_choices:
@@ -332,13 +368,13 @@ class AMDSMIParser(argparse.ArgumentParser):
                     setattr(args, self.dest, selected_device_handles)
                 else:
                     if selected_device_handles == '':
-                        raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException("--cpu", _CPUSelectAction.ouputformat)
+                        raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException("--cpu", _CPUSelectAction.outputformat)
                     elif not cpu_format:
                         raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(selected_device_handles,
-                                                                                         _CPUSelectAction.ouputformat)
+                                                                                         _CPUSelectAction.outputformat)
                     else:
                         raise amdsmi_cli_exceptions.AmdSmiDeviceNotFoundException(selected_device_handles,
-                                                                                  _CPUSelectAction.ouputformat,
+                                                                                  _CPUSelectAction.outputformat,
                                                                                   False, True, False)
         return _CPUSelectAction
 
@@ -350,7 +386,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         """
         amdsmi_helpers = self.helpers
         class _CoreSelectAction(argparse.Action):
-            ouputformat=self.helpers.get_output_format()
+            outputformat=self.helpers.get_output_format()
             # Checks the values
             def __call__(self, parser, args, values, option_string=None):
                 if "all" in core_choices:
@@ -361,18 +397,18 @@ class AMDSMIParser(argparse.ArgumentParser):
                     setattr(args, self.dest, selected_device_handles)
                 else:
                     if selected_device_handles == '':
-                        raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException("--core", _CoreSelectAction.ouputformat)
+                        raise amdsmi_cli_exceptions.AmdSmiMissingParameterValueException("--core", _CoreSelectAction.outputformat)
                     elif not core_format:
                         raise amdsmi_cli_exceptions.AmdSmiInvalidParameterValueException(selected_device_handles,
-                                                                                         _CoreSelectAction.ouputformat)
+                                                                                         _CoreSelectAction.outputformat)
                     else:
                         raise amdsmi_cli_exceptions.AmdSmiDeviceNotFoundException(selected_device_handles,
-                                                                                  _CoreSelectAction.ouputformat,
+                                                                                  _CoreSelectAction.outputformat,
                                                                                   False, False, True)
         return _CoreSelectAction
 
 
-    def _add_command_modifiers(self, subcommand_parser):
+    def _add_command_modifiers(self, subcommand_parser: argparse.ArgumentParser):
         json_help = "Displays output in JSON format (human readable by default)."
         csv_help = "Displays output in CSV format (human readable by default)."
         file_help = "Saves output into a file on the provided path (stdout by default)."
@@ -401,11 +437,11 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         # Mutually Exclusive Args within the subparser
         subcommand_parser.add_argument('-w', '--watch', action='store', metavar='INTERVAL',
-             type=self._positive_int, required=False, help=watch_help)
+             type=lambda value: self._positive_int(value, '--watch'), required=False, help=watch_help)
         subcommand_parser.add_argument('-W', '--watch_time', action=self._check_watch_selected(), metavar='TIME',
-            type=self._positive_int, required=False, help=watch_time_help)
+            type=lambda value: self._positive_int(value, '--watch_time'), required=False, help=watch_time_help)
         subcommand_parser.add_argument('-i', '--iterations', action=self._check_watch_selected(), metavar='ITERATIONS',
-            type=self._positive_int, required=False, help=iterations_help)
+            type=lambda value: self._positive_int(value, '--iterations'), required=False, help=iterations_help)
 
 
     def _validate_cpu_core(self, value):
@@ -431,7 +467,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         return value
 
 
-    def _add_device_arguments(self, subcommand_parser, required=False):
+    def _add_device_arguments(self, subcommand_parser: argparse.ArgumentParser, required=False):
         # Device arguments help text
         gpu_help = f"Select a GPU ID, BDF, or UUID from the possible choices:\n{self.gpu_choices_str}"
         vf_help = "Gets general information about the specified VF (timeslice, fb info, …).\
@@ -554,7 +590,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         return _ValidateOverdrivePercent
 
 
-    def _add_version_parser(self, subparsers, func):
+    def _add_version_parser(self, subparsers: argparse._SubParsersAction, func):
         # Subparser help text
         version_help = "Display version information"
 
@@ -567,8 +603,16 @@ class AMDSMIParser(argparse.ArgumentParser):
         # Add Universal Arguments
         self._add_command_modifiers(version_parser)
 
+        # help info
+        gpu_version_help = "Display the current amdgpu driver version"
+        cpu_version_help = "Display the current amd_hsmp driver version"
 
-    def _add_list_parser(self, subparsers, func):
+        # Add GPU and CPU version Arguments
+        version_parser.add_argument('-g', '--gpu_version', action='store_true', required=False, help=gpu_version_help, default=None)
+        version_parser.add_argument('-c', '--cpu_version', action='store_true', required=False, help=cpu_version_help, default=None)
+
+
+    def _add_list_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_amdgpu_initialized():
             # The list subcommand is only applicable to systems with amdgpu initialized
             return
@@ -582,6 +626,8 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         # Create list subparser
         list_parser = subparsers.add_parser('list', help=list_help, description=list_subcommand_help)
+        # Create -e subparser
+        list_parser.add_argument("-e", action="store_true", help="Enumeration mapping to other features.\n    Lists the BDF, UUID, KFD_ID, CARD, RENDER, HIP_ID, HIP_UUID and HSA_ID for each GPU.")
         list_parser.formatter_class=lambda prog: AMDSMISubparserHelpFormatter(prog)
         list_parser.set_defaults(func=func)
 
@@ -590,7 +636,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_device_arguments(list_parser, required=False)
 
 
-    def _add_static_parser(self, subparsers, func):
+    def _add_static_parser(self, subparsers: argparse._SubParsersAction, func):
         # Subparser help text
         static_help = "Gets static information about the specified GPU"
         static_subcommand_help = "If no GPU is specified, returns static information for all GPUs on the system.\
@@ -609,6 +655,10 @@ class AMDSMIParser(argparse.ArgumentParser):
         soc_pstate_help = "The available soc pstate policy"
         xgmi_plpd_help = "The available XGMI per-link power down policy"
         process_isolation_help = "The process isolation status"
+        clk_options = self.helpers.get_clock_types()[0]
+        clk_options.remove('PCIE')
+        clk_option_str = ", ".join(clk_options) + ", ALL"
+        clock_help = f"Show one or more valid clock frequency levels. Available options:\n\t{clk_option_str}"
 
         # Options arguments help text for Hypervisors and Baremetal
         ras_help = "Displays RAS features information"
@@ -644,10 +694,11 @@ class AMDSMIParser(argparse.ArgumentParser):
             static_parser.add_argument('-B', '--board', action='store_true', required=False, help=board_help)
             static_parser.add_argument('-R', '--process-isolation', action='store_true', required=False, help=process_isolation_help)
             static_parser.add_argument('-r', '--ras', action='store_true', required=False, help=ras_help)
+            static_parser.add_argument('-C', '--clock', action='store', default=False, nargs='*', type=str, required=False, help=clock_help)
+            static_parser.add_argument('-p', '--partition', action='store_true', required=False, help=partition_help)
 
             # Options to display on Hypervisors and Baremetal
             if self.helpers.is_hypervisor() or self.helpers.is_baremetal():
-                static_parser.add_argument('-p', '--partition', action='store_true', required=False, help=partition_help)
                 static_parser.add_argument('-l', '--limit', action='store_true', required=False, help=limit_help)
                 static_parser.add_argument('-P', '--soc-pstate', action='store_true', required=False, help=soc_pstate_help)
                 static_parser.add_argument('-x', '--xgmi-plpd', action='store_true', required=False, help=xgmi_plpd_help)
@@ -767,7 +818,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         perf_level_help = "Current DPM performance level"
         xgmi_err_help = "XGMI error information since last read"
         energy_help = "Amount of energy consumed"
-        throttle_help = "Displays throttle accumulators; Only available for MI300 or newer ASICs"
+        throttle_help = "Displays throttle accumulators;\n    Only available for MI300 or newer ASICs"
 
         # Help text for Arguments only on Hypervisors
         schedule_help = "All scheduling information"
@@ -782,7 +833,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         cpu_freq_help = "Displays currentFclkMemclk frequencies and cclk frequency limit"
         cpu_c0_res_help = "Displays C0 residency"
         cpu_lclk_dpm_help = "Displays lclk dpm level range. Requires socket ID and NBOID as inputs"
-        cpu_pwr_svi_telemtry_rails_help = "Displays svi based telemetry for all rails"
+        cpu_pwr_svi_telemetry_rails_help = "Displays svi based telemetry for all rails"
         cpu_io_bandwidth_help = "Displays current IO bandwidth for the selected CPU.\
         \n input parameters are bandwidth type(1) and link ID encodings\
         \n i.e. P2, P3, G0 - G7"
@@ -862,8 +913,8 @@ class AMDSMIParser(argparse.ArgumentParser):
             cpu_group.add_argument('--cpu-c0-res', action='store_true', required=False, help=cpu_c0_res_help)
             cpu_group.add_argument('--cpu-lclk-dpm-level', action='append', required=False, type=self._not_negative_int,
                                     nargs=1, metavar=("NBIOID"), help=cpu_lclk_dpm_help)
-            cpu_group.add_argument('--cpu-pwr-svi-telemtry-rails', action='store_true', required=False,
-                                    help=cpu_pwr_svi_telemtry_rails_help)
+            cpu_group.add_argument('--cpu-pwr-svi-telemetry-rails', action='store_true', required=False,
+                                    help=cpu_pwr_svi_telemetry_rails_help)
             cpu_group.add_argument('--cpu-io-bandwidth', action='append', required=False, nargs=2,
                                     metavar=("IO_BW", "LINKID_NAME"), help=cpu_io_bandwidth_help)
             cpu_group.add_argument('--cpu-xgmi-bandwidth', action='append', required=False, nargs=2,
@@ -891,7 +942,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_command_modifiers(metric_parser)
 
 
-    def _add_process_parser(self, subparsers, func):
+    def _add_process_parser(self, subparsers: argparse._SubParsersAction, func):
         if self.helpers.is_hypervisor():
             # Don't add this subparser on Hypervisors
             # This subparser is only available to Guest and Baremetal systems
@@ -931,11 +982,11 @@ class AMDSMIParser(argparse.ArgumentParser):
         # Optional Args
         process_parser.add_argument('-G', '--general', action='store_true', required=False, help=general_help)
         process_parser.add_argument('-e', '--engine', action='store_true', required=False, help=engine_help)
-        process_parser.add_argument('-p', '--pid', action='store', type=self._not_negative_int, required=False, help=pid_help)
-        process_parser.add_argument('-n', '--name', action='store', type=self._is_valid_string, required=False, help=name_help)
+        process_parser.add_argument('-p', '--pid', action='store', type=lambda value: self._not_negative_int(value, '--pid'), required=False, help=pid_help)
+        process_parser.add_argument('-n', '--name', action='store', type=lambda value: self._is_valid_string(value, '--name'), required=False, help=name_help)
 
 
-    def _add_profile_parser(self, subparsers, func):
+    def _add_profile_parser(self, subparsers: argparse._SubParsersAction, func):
         if not (self.helpers.is_windows() and self.helpers.is_hypervisor()):
             # This subparser only applies to Hypervisors
             return
@@ -956,7 +1007,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_device_arguments(profile_parser, required=False)
 
 
-    def _add_event_parser(self, subparsers, func):
+    def _add_event_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_amdgpu_initialized():
             # The event subcommand is only applicable to systems with amdgpu initialized
             return
@@ -977,7 +1028,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_device_arguments(event_parser, required=False)
 
 
-    def _add_topology_parser(self, subparsers, func):
+    def _add_topology_parser(self, subparsers: argparse._SubParsersAction, func):
         if not(self.helpers.is_baremetal() and self.helpers.is_linux()):
             # This subparser is only applicable to Baremetal Linux
             return
@@ -1025,7 +1076,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         topology_parser.add_argument('-z', '--bi-dir', action='store_true', required=False, help=bi_dir_help)
 
 
-    def _add_set_value_parser(self, subparsers, func):
+    def _add_set_value_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_linux():
             # This subparser is only applicable to Linux
             return
@@ -1037,19 +1088,30 @@ class AMDSMIParser(argparse.ArgumentParser):
         set_value_optionals_title = "Set Arguments"
 
         # Help text for Arguments only on BM platforms
-        set_fan_help = "Set GPU fan speed (0-255 or 0-100%%)"
-        set_perf_level_help = "Set performance level"
-        set_profile_help = "Set power profile level (#) or a quoted string of custom profile attributes"
-        set_perf_det_help = "Set GPU clock frequency limit and performance level to determinism\n to get minimal performance variation"
-        compute_partition_choices_str = ", ".join(self.helpers.get_compute_partition_types())
-        memory_partition_choices_str = ", ".join(self.helpers.get_memory_partition_types())
-        set_compute_partition_help = f"Set one of the following the compute partition modes:\n\t{compute_partition_choices_str}"
-        set_memory_partition_help = f"Set one of the following the memory partition modes:\n\t{memory_partition_choices_str}"
-        set_power_cap_help = "Set power capacity limit"
-        set_soc_pstate_help = "Set the GPU soc pstate policy using policy id\n"
-        set_xgmi_plpd_help = "Set the GPU XGMI per-link power down policy using policy id\n"
-        set_clk_limit_help = "Sets the sclk (aka gfxclk) or mclk minimum and maximum frequencies:\n\tamd-smi set -L (sclk | mclk) (min | max) value"
-        set_process_isolation_help = "Enable or disable the GPU process isolation on a per partition basis:\n\t0 for disable and 1 for enable.\n"
+        if self.helpers.is_amdgpu_initialized():
+            if self.helpers.is_baremetal():
+                set_fan_help = "Set GPU fan speed (0-255 or 0-100%%)"
+                perf_level_help_choices_str = ", ".join(self.helpers.get_perf_levels()[0][0:-1])
+                set_perf_level_help = f"Set one of the following performance levels:\n\t{perf_level_help_choices_str}"
+                power_profile_choices_str = ", ".join(self.helpers.get_power_profiles()[0:-1])
+                set_profile_help = f"Set power profile level (#) or choose one of available profiles:\n\t{power_profile_choices_str}"
+                perf_det_choices_str = ", ".join(self.helpers.get_perf_det_levels())
+                set_perf_det_help = f"Set performance determinism and select one of the corresponding performance levels:\n\t{perf_det_choices_str}"
+                (accelerator_set_choices, _) = self.helpers.get_accelerator_choices_types_indices()
+                memory_partition_choices_str = ", ".join(self.helpers.get_memory_partition_types())
+                set_compute_partition_help = f"Set one of the following the accelerator type or profile index:\n\t{accelerator_set_choices}.\n\tUse `sudo amd-smi partition --accelerator` to find acceptable values."
+                set_memory_partition_help = f"Set one of the following the memory partition modes:\n\t{memory_partition_choices_str}"
+                power_cap_min, power_cap_max = self.helpers.get_power_caps()
+                power_cap_max = self.helpers.convert_SI_unit(power_cap_max, AMDSMIHelpers.SI_Unit.MICRO)
+                power_cap_min = self.helpers.convert_SI_unit(power_cap_min, AMDSMIHelpers.SI_Unit.MICRO)
+                set_power_cap_help = f"Set power capacity limit:\n\tmin cap: {power_cap_min} W, max cap: {power_cap_max} W"
+                soc_pstate_help_info = ", ".join(self.helpers.get_soc_pstates())
+                set_soc_pstate_help = f"Set the GPU soc pstate policy using policy id, an integer. Valid id's include:\n\t{soc_pstate_help_info}"
+                xgmi_plpd_help_info = ", ".join(self.helpers.get_xgmi_plpd_policies())
+                set_xgmi_plpd_help = f"Set the GPU XGMI per-link power down policy using policy id, an integer. Valid id's include:\n\t{xgmi_plpd_help_info}"
+                set_clock_freq_help = "Set one or more sclk (aka gfxclk), mclk, fclk, pcie, or socclk frequency levels.\n\tUse `amd-smi static --clock` to find acceptable levels."
+            set_clk_limit_help = "Sets the sclk (aka gfxclk) or mclk minimum and maximum frequencies. \n\tex: amd-smi set -L (sclk | mclk) (min | max) value"
+            set_process_isolation_help = "Enable or disable the GPU process isolation on a per partition basis: 0 for disable and 1 for enable.\n"
 
         # Help text for CPU set options
         set_cpu_pwr_limit_help = "Set power limit for the given socket. Input parameter is power limit value."
@@ -1077,20 +1139,23 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_device_arguments(set_value_parser, required=False)
 
         if self.helpers.is_amdgpu_initialized():
+            # set value should only take one of these at a time so args below will be mutually exclusive
+            set_value_exclusive_group = set_value_parser.add_mutually_exclusive_group()
             if self.helpers.is_baremetal():
                 # Optional GPU Args
-                set_value_parser.add_argument('-f', '--fan', action=self._validate_fan_speed(), required=False, help=set_fan_help, metavar='%')
-                set_value_parser.add_argument('-l', '--perf-level', action='store', choices=self.helpers.get_perf_levels()[0], type=str.upper, required=False, help=set_perf_level_help, metavar='LEVEL')
-                set_value_parser.add_argument('-P', '--profile', action='store', required=False, help=set_profile_help, metavar='SETPROFILE')
-                set_value_parser.add_argument('-d', '--perf-determinism', action='store', type=self._not_negative_int, required=False, help=set_perf_det_help, metavar='SCLKMAX')
-                set_value_parser.add_argument('-C', '--compute-partition', action='store', choices=self.helpers.get_compute_partition_types(), type=str.upper, required=False, help=set_compute_partition_help, metavar='PARTITION')
-                set_value_parser.add_argument('-M', '--memory-partition', action='store', choices=self.helpers.get_memory_partition_types(), type=str.upper, required=False, help=set_memory_partition_help, metavar='PARTITION')
-                set_value_parser.add_argument('-o', '--power-cap', action='store', type=self._positive_int, required=False, help=set_power_cap_help, metavar='WATTS')
-                set_value_parser.add_argument('-p', '--soc-pstate', action='store', required=False, type=self._not_negative_int, help=set_soc_pstate_help, metavar='POLICY_ID')
-                set_value_parser.add_argument('-x', '--xgmi-plpd', action='store', required=False, type=self._not_negative_int, help=set_xgmi_plpd_help, metavar='POLICY_ID')
+                set_value_exclusive_group.add_argument('-f', '--fan', action=self._validate_fan_speed(), required=False, help=set_fan_help, metavar='%')
+                set_value_exclusive_group.add_argument('-l', '--perf-level', action='store', choices=self.helpers.get_perf_levels()[0], type=str.upper, required=False, help=set_perf_level_help, metavar='LEVEL')
+                set_value_exclusive_group.add_argument('-P', '--profile', action='store', required=False, help=set_profile_help, metavar='SETPROFILE')
+                set_value_exclusive_group.add_argument('-d', '--perf-determinism', action='store', type=lambda value: self._not_negative_int(value, '--perf-determinism'), required=False, help=set_perf_det_help, metavar='SCLKMAX')
+                set_value_exclusive_group.add_argument('-C', '--compute-partition', action='store', choices=accelerator_set_choices, type=lambda value: self._is_command_supported(value, accelerator_set_choices, '--compute-partition'), required=False, help=set_compute_partition_help, metavar='<ACCELERATOR_TYPE> or <PROFILE_INDEX>')
+                set_value_exclusive_group.add_argument('-M', '--memory-partition', action='store', choices=self.helpers.get_memory_partition_types(), type=str.upper, required=False, help=set_memory_partition_help, metavar='PARTITION')
+                set_value_exclusive_group.add_argument('-o', '--power-cap', action='store', type=lambda value: self._positive_int(value, '--power-cap'), required=False, help=set_power_cap_help, metavar='WATTS')
+                set_value_exclusive_group.add_argument('-p', '--soc-pstate', action='store', required=False, type=lambda value: self._not_negative_int(value, '--soc-pstate'), help=set_soc_pstate_help, metavar='POLICY_ID')
+                set_value_exclusive_group.add_argument('-x', '--xgmi-plpd', action='store', required=False, type=lambda value: self._not_negative_int(value, '--xgmi-plpd'), help=set_xgmi_plpd_help, metavar='POLICY_ID')
+                set_value_exclusive_group.add_argument('-c', '--clk-level', action=self._level_select(), nargs='+', required=False, help=set_clock_freq_help, metavar=('CLK_TYPE', 'FREQ_LEVELS'))
+            set_value_exclusive_group.add_argument('-L', '--clk-limit', action=self._limit_select(), nargs=3, required=False, help=set_clk_limit_help, metavar=('CLK_TYPE', 'LIM_TYPE', 'VALUE'))
+            set_value_exclusive_group.add_argument('-R', '--process-isolation', action='store', choices=[0,1], type=lambda value: self._not_negative_int(value, '--process-isolation'), required=False, help=set_process_isolation_help, metavar='STATUS')
 
-            set_value_parser.add_argument('-L', '--clk-limit', action=self._limit_select(), nargs=3, required=False, help=set_clk_limit_help, metavar=('CLK_TYPE', 'LIM_TYPE', 'VALUE'))
-            set_value_parser.add_argument('-R', '--process-isolation', action='store', choices=[0,1], type=self._not_negative_int, required=False, help=set_process_isolation_help, metavar='STATUS')
 
         if self.helpers.is_amd_hsmp_initialized():
             if self.helpers.is_baremetal():
@@ -1115,7 +1180,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         self._add_command_modifiers(set_value_parser)
 
 
-    def _add_reset_parser(self, subparsers, func):
+    def _add_reset_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_linux():
             # This subparser is only applicable to Linux
             return
@@ -1151,21 +1216,24 @@ class AMDSMIParser(argparse.ArgumentParser):
         # Providing no -g 0 or -g all, is not required
         self._add_device_arguments(reset_parser, required=False)
 
+        # make reset args mutually exclusive
+        reset_exclusive_group = reset_parser.add_mutually_exclusive_group()
+
         if self.helpers.is_baremetal():
             # Add Baremetal reset arguments
-            reset_parser.add_argument('-G', '--gpureset', action='store_true', required=False, help=gpureset_help)
-            reset_parser.add_argument('-c', '--clocks', action='store_true', required=False, help=reset_clocks_help)
-            reset_parser.add_argument('-f', '--fans', action='store_true', required=False, help=reset_fans_help)
-            reset_parser.add_argument('-p', '--profile', action='store_true', required=False, help=reset_profile_help)
-            reset_parser.add_argument('-x', '--xgmierr', action='store_true', required=False, help=reset_xgmierr_help)
-            reset_parser.add_argument('-d', '--perf-determinism', action='store_true', required=False, help=reset_perf_det_help)
-            reset_parser.add_argument('-o', '--power-cap', action='store_true', required=False, help=reset_power_cap_help)
+            reset_exclusive_group.add_argument('-G', '--gpureset', action='store_true', required=False, help=gpureset_help)
+            reset_exclusive_group.add_argument('-c', '--clocks', action='store_true', required=False, help=reset_clocks_help)
+            reset_exclusive_group.add_argument('-f', '--fans', action='store_true', required=False, help=reset_fans_help)
+            reset_exclusive_group.add_argument('-p', '--profile', action='store_true', required=False, help=reset_profile_help)
+            reset_exclusive_group.add_argument('-x', '--xgmierr', action='store_true', required=False, help=reset_xgmierr_help)
+            reset_exclusive_group.add_argument('-d', '--perf-determinism', action='store_true', required=False, help=reset_perf_det_help)
+            reset_exclusive_group.add_argument('-o', '--power-cap', action='store_true', required=False, help=reset_power_cap_help)
 
         # Add Baremetal and Virtual OS reset arguments
-        reset_parser.add_argument('-l', '--clean-local-data', action='store_true', required=False, help=reset_gpu_clean_local_data_help)
+        reset_exclusive_group.add_argument('-l', '--clean-local-data', action='store_true', required=False, help=reset_gpu_clean_local_data_help)
 
 
-    def _add_monitor_parser(self, subparsers, func):
+    def _add_monitor_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_linux():
             # This subparser is only applicable to Linux
             return
@@ -1192,7 +1260,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         mem_usage_help = "Monitor memory usage in MB"
         pcie_bandwidth_help = "Monitor PCIe bandwidth in Mb/s"
         process_help = "Enable Process information table below monitor output"
-        violation_help = "Monitor power and thermal violation status (%%); Only available for MI300 or newer ASICs"
+        violation_help = "Monitor power and thermal violation status (%%);\n    Only available for MI300 or newer ASICs"
 
         # Create monitor subparser
         monitor_parser = subparsers.add_parser('monitor', help=monitor_help, description=monitor_subcommand_help, aliases=["dmon"])
@@ -1264,7 +1332,7 @@ class AMDSMIParser(argparse.ArgumentParser):
         rocm_smi_parser.add_argument('-f', '--showclkfrq', action='store_true', required=False, help=showclkfrq_help)
 
 
-    def _add_xgmi_parser(self, subparsers, func):
+    def _add_xgmi_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_amdgpu_initialized():
             # The xgmi subcommand is only applicable to systems with amdgpu initialized
             return
@@ -1277,6 +1345,7 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         # Help text for Arguments only on Guest and BM platforms
         metrics_help = "Metric XGMI information"
+        xgmi_link_status_help = "XGMI Link Status information"
 
         # Create xgmi subparser
         xgmi_parser = subparsers.add_parser('xgmi', help=xgmi_help, description=xgmi_subcommand_help)
@@ -1290,9 +1359,10 @@ class AMDSMIParser(argparse.ArgumentParser):
 
         # Optional Args
         xgmi_parser.add_argument('-m', '--metric', action='store_true', required=False, help=metrics_help)
+        xgmi_parser.add_argument('-l', '--link-status', action='store_true', required=False, help=xgmi_link_status_help)
 
 
-    def _add_partition_parser(self, subparsers, func):
+    def _add_partition_parser(self, subparsers: argparse._SubParsersAction, func):
         if not self.helpers.is_amdgpu_initialized():
             # The partition subcommand is only applicable to systems with amdgpu initialized
             return
