@@ -1,49 +1,29 @@
 /*
- * =============================================================================
- * The University of Illinois/NCSA
- * Open Source License (NCSA)
- *
- * Copyright (c) 2024, Advanced Micro Devices, Inc.
- * All rights reserved.
- *
- * Developed by:
- *
- *                 AMD Research and AMD ROC Software Development
- *
- *                 Advanced Micro Devices, Inc.
- *
- *                 www.amd.com
+ * Copyright (c) Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal with the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimers.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimers in
- *    the documentation and/or other materials provided with the distribution.
- *  - Neither the names of <Name of Development Group, Name of Institution>,
- *    nor the names of its contributors may be used to endorse or promote
- *    products derived from this Software without specific prior written
- *    permission.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
- * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
- * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS WITH THE SOFTWARE.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 #include <pwd.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <inttypes.h>
 
 #include <bitset>
 #include <cassert>
@@ -54,6 +34,7 @@
 #include <sstream>
 
 #include "amd_smi/amdsmi.h"
+#include "amd_smi/impl/amd_smi_utils.h"
 
 
 #define CHK_AMDSMI_RET(RET)                                                    \
@@ -221,8 +202,62 @@ std::string print_unsigned_int(T value) {
   return ss.str();
 }
 
+static const std::string
+computePartitionString(amdsmi_compute_partition_type_t computeParitionType) {
+  switch (computeParitionType) {
+    case AMDSMI_COMPUTE_PARTITION_SPX:
+      return "SPX";
+    case AMDSMI_COMPUTE_PARTITION_DPX:
+      return "DPX";
+    case AMDSMI_COMPUTE_PARTITION_TPX:
+      return "TPX";
+    case AMDSMI_COMPUTE_PARTITION_QPX:
+      return "QPX";
+    case AMDSMI_COMPUTE_PARTITION_CPX:
+      return "CPX";
+    default:
+      return "N/A";
+  }
+}
+
+static const std::map<std::string, amdsmi_compute_partition_type_t>
+mapStringToSMIComputePartitionTypes {
+  {"SPX", AMDSMI_COMPUTE_PARTITION_SPX},
+  {"DPX", AMDSMI_COMPUTE_PARTITION_DPX},
+  {"TPX", AMDSMI_COMPUTE_PARTITION_TPX},
+  {"QPX", AMDSMI_COMPUTE_PARTITION_QPX},
+  {"CPX", AMDSMI_COMPUTE_PARTITION_CPX},
+  {"N/A", AMDSMI_COMPUTE_PARTITION_INVALID}
+};
+
+static const std::string
+memoryPartitionString(amdsmi_memory_partition_type_t memoryParitionType) {
+  switch (memoryParitionType) {
+    case AMDSMI_MEMORY_PARTITION_NPS1:
+      return "NPS1";
+    case AMDSMI_MEMORY_PARTITION_NPS2:
+      return "NPS2";
+    case AMDSMI_MEMORY_PARTITION_NPS4:
+      return "NPS4";
+    case AMDSMI_MEMORY_PARTITION_NPS8:
+      return "NPS8";
+    default:
+      return "N/A";
+  }
+}
+
+static const std::map<std::string, amdsmi_memory_partition_type_t>
+mapStringToSMIMemoryPartitionTypes {
+  {"NPS1", AMDSMI_MEMORY_PARTITION_NPS1},
+  {"NPS2", AMDSMI_MEMORY_PARTITION_NPS2},
+  {"NPS4", AMDSMI_MEMORY_PARTITION_NPS4},
+  {"NPS8", AMDSMI_MEMORY_PARTITION_NPS8},
+  {"N/A", AMDSMI_MEMORY_PARTITION_UNKNOWN}
+};
+
 int main() {
-    amdsmi_status_t ret;
+    amdsmi_status_t ret, ret_set;
+    const char *err_str;
 
     // Init amdsmi for sockets and devices.
     // Here we are only interested in AMD_GPUS.
@@ -268,6 +303,20 @@ int main() {
 
         // For each device of the socket, get name and temperature.
         for (uint32_t j = 0; j < device_count; j++) {
+            uint32_t device_cnt = 0;
+            ret = smi_amdgpu_get_device_count(&device_cnt);
+            CHK_AMDSMI_RET(ret)
+            std::cout << "Device Count: " << device_cnt << std::endl;
+
+            // Get device index
+            uint32_t device_index = 0;
+            ret = smi_amdgpu_get_device_index(processor_handles[j], &device_index);
+            CHK_AMDSMI_RET(ret)
+            std::cout << "Device Index: " << device_index << std::endl;
+
+            std::vector<amdsmi_processor_handle> p_handles(device_cnt);
+            ret = smi_amdgpu_get_processor_handle_by_index(device_index, &p_handles[j]);
+
             // Get device type. Since the amdsmi is initialized with
             // AMD_SMI_INIT_AMD_GPUS, the processor_type must be AMDSMI_PROCESSOR_TYPE_AMD_GPU.
             processor_type_t processor_type = {};
@@ -283,11 +332,11 @@ int main() {
             ret = amdsmi_get_gpu_device_bdf(processor_handles[j], &bdf);
             CHK_AMDSMI_RET(ret)
             printf("    Output of amdsmi_get_gpu_device_bdf:\n");
-            printf("\tDevice[%d] BDF %04lx:%02x:%02x.%d\n\n", i,
-                   bdf.domain_number,
-                   bdf.bus_number,
-                   bdf.device_number,
-                   bdf.function_number);
+            printf("\tDevice[%d] BDF %04" PRIx64 ":%02" PRIx32 ":%02" PRIx32 ".%" PRIu32 "\n\n", i,
+                   static_cast<uint64_t>(bdf.domain_number),
+                   static_cast<uint32_t>(bdf.bus_number),
+                   static_cast<uint32_t>(bdf.device_number),
+                   static_cast<uint32_t>(bdf.function_number));
 
             // Get handle from BDF
             amdsmi_processor_handle dev_handle;
@@ -306,6 +355,173 @@ int main() {
             printf("\tAsic serial: 0x%s\n", asic_info.asic_serial);
             printf("\tNum of Computes: %d\n\n", asic_info.num_of_compute_units);
 
+            bool is_power_management_enabled = false;
+            ret = amdsmi_is_gpu_power_management_enabled(processor_handles[j],
+                                                    &is_power_management_enabled);
+            CHK_AMDSMI_RET(ret)
+            printf("    Output of amdsmi_is_gpu_power_management_enabled:\n");
+            printf("\tPower Management Enabled: %s\n\n",
+                    (is_power_management_enabled ? "TRUE" : "FALSE"));
+
+            std::cout << "    **Version 1: Accelerator/Compute Partition API Examples**\n";
+            char original_compute_partition[AMDSMI_MAX_STRING_LENGTH];
+            ret = amdsmi_get_gpu_compute_partition(processor_handles[j], original_compute_partition,
+                                        static_cast<uint32_t>(AMDSMI_MAX_STRING_LENGTH));
+
+            amdsmi_status_code_to_string(ret, &err_str);
+            if (ret == AMDSMI_STATUS_SUCCESS) {
+                CHK_AMDSMI_RET(ret)
+                std::cout << "    Output of amdsmi_get_gpu_compute_partition:\n";
+                std::cout << "\tamdsmi_get_gpu_compute_partition(" << j << ", "
+                << mapStringToSMIComputePartitionTypes.at(original_compute_partition) << "): "
+                << err_str << "\n\n";
+                std::cout << "\tCompute Partition (original): "
+                << original_compute_partition << "\n\n";
+            } else {
+                std::cout << "\tamdsmi_get_gpu_compute_partition(" << j << ", "
+                << computePartitionString(AMDSMI_COMPUTE_PARTITION_INVALID) << "): "
+                << err_str << "\n\n";
+            }
+
+            for (int partition = static_cast<int>(AMDSMI_COMPUTE_PARTITION_SPX);
+                    partition <= static_cast<int>(AMDSMI_COMPUTE_PARTITION_CPX);
+                    partition++) {
+                amdsmi_compute_partition_type_t updatePartition
+                    = static_cast<amdsmi_compute_partition_type_t>(partition);
+                ret_set = amdsmi_set_gpu_compute_partition(processor_handles[j],
+                                                                updatePartition);
+                amdsmi_status_code_to_string(ret_set, &err_str);
+                if (ret_set == AMDSMI_STATUS_SUCCESS) {
+                    CHK_AMDSMI_RET(ret_set)
+                }
+                std::cout << "\tamdsmi_set_gpu_compute_partition(" << j << ", "
+                << computePartitionString(updatePartition) << "): "
+                << err_str << "\n\n";
+
+                // Get the current compute partition
+                char current_compute_partition[AMDSMI_MAX_STRING_LENGTH];
+                ret = amdsmi_get_gpu_compute_partition(processor_handles[j],
+                                                current_compute_partition,
+                                                static_cast<uint32_t>(AMDSMI_MAX_STRING_LENGTH));
+                amdsmi_status_code_to_string(ret, &err_str);
+                if (ret == AMDSMI_STATUS_SUCCESS) {
+                    CHK_AMDSMI_RET(ret)
+                    std::cout << "    Output of amdsmi_get_gpu_compute_partition:\n";
+                    std::cout << "\tamdsmi_get_gpu_compute_partition(" << j << ", "
+                              << computePartitionString(updatePartition) << "): "
+                              << err_str << "\n\n";
+                    std::cout << "\tCompute Partition (current): "
+                              << current_compute_partition << "\n\n";
+                } else {
+                    std::cout << "\tamdsmi_get_gpu_compute_partition(" << j << ", "
+                    << computePartitionString(AMDSMI_COMPUTE_PARTITION_INVALID) << "): "
+                    << err_str << "\n\n";
+                }
+            }
+            // return to original compute partition
+            amdsmi_compute_partition_type_t original_compute_partition_type;
+            if (ret == AMDSMI_STATUS_SUCCESS) {
+                original_compute_partition_type
+                    = mapStringToSMIComputePartitionTypes.at(original_compute_partition);
+            } else {
+                original_compute_partition_type = AMDSMI_COMPUTE_PARTITION_INVALID;
+            }
+            std::cout << "    Returning to original compute partition ("
+                      << computePartitionString(original_compute_partition_type) << ")\n";
+            auto ret_set = amdsmi_set_gpu_compute_partition(processor_handles[j],
+                                                original_compute_partition_type);
+            amdsmi_status_code_to_string(ret_set, &err_str);
+            if (ret_set == AMDSMI_STATUS_SUCCESS) {
+                CHK_AMDSMI_RET(ret_set)
+            }
+            std::cout << "\tamdsmi_set_gpu_compute_partition(" << j << ", "
+            << computePartitionString(original_compute_partition_type) << "): "
+            << err_str << "\n\n";
+
+            std::cout << "    **Version 1: Memory Partition API Examples**\n";
+            char original_memory_partition[AMDSMI_MAX_STRING_LENGTH];
+            ret = amdsmi_get_gpu_memory_partition(processor_handles[j], original_memory_partition,
+                                        static_cast<uint32_t>(AMDSMI_MAX_STRING_LENGTH));
+            amdsmi_status_code_to_string(ret, &err_str);
+            if (ret == AMDSMI_STATUS_SUCCESS) {
+                CHK_AMDSMI_RET(ret)
+                std::cout << "    Output of amdsmi_get_gpu_memory_partition:\n";
+                std::cout << "\tamdsmi_get_gpu_memory_partition(" << j << ", "
+                << mapStringToSMIMemoryPartitionTypes.at(original_memory_partition) << "): "
+                << err_str << "\n\n";
+                std::cout << "\tMemory Partition (original): "
+                << original_memory_partition << "\n\n";
+            } else {
+                std::cout << "\tamdsmi_get_gpu_memory_partition(" << j << ", "
+                << memoryPartitionString(AMDSMI_MEMORY_PARTITION_UNKNOWN) << "): "
+                << err_str << "\n\n";
+            }
+
+            for (int partition = static_cast<int>(AMDSMI_MEMORY_PARTITION_NPS1);
+                    partition <= static_cast<int>(AMDSMI_MEMORY_PARTITION_NPS8);
+                    partition++) {
+                if (partition != static_cast<int>(AMDSMI_MEMORY_PARTITION_NPS1)
+                    && partition != static_cast<int>(AMDSMI_MEMORY_PARTITION_NPS2)
+                    && partition != static_cast<int>(AMDSMI_MEMORY_PARTITION_NPS4)
+                    && partition != static_cast<int>(AMDSMI_MEMORY_PARTITION_NPS8)) {
+                    continue;
+                }
+                amdsmi_memory_partition_type_t updatePartition
+                    = static_cast<amdsmi_memory_partition_type_t>(partition);
+                auto ret_set = amdsmi_set_gpu_memory_partition(processor_handles[j],
+                                                                updatePartition);
+                amdsmi_status_code_to_string(ret_set, &err_str);
+                if (ret_set == AMDSMI_STATUS_SUCCESS) {
+                    CHK_AMDSMI_RET(ret_set)
+                    std::cout << "    Output of amdsmi_set_gpu_memory_partition:\n";
+                }
+                std::cout << "\tamdsmi_set_gpu_memory_partition(" << j << ", "
+                          << memoryPartitionString(updatePartition) << "): "
+                          << err_str << "\n\n";
+
+                // Get the current memory partition
+                char current_memory_partition[AMDSMI_MAX_STRING_LENGTH];
+                ret = amdsmi_get_gpu_memory_partition(processor_handles[j],
+                                            current_memory_partition,
+                                            static_cast<uint32_t>(AMDSMI_MAX_STRING_LENGTH));
+
+                amdsmi_status_code_to_string(ret, &err_str);
+                if (ret == AMDSMI_STATUS_SUCCESS) {
+                    CHK_AMDSMI_RET(ret)
+                    std::cout << "\tamdsmi_get_gpu_memory_partition(" << j << ", "
+                    << memoryPartitionString(updatePartition) << "): "
+                    << err_str << "\n\n";
+                    std::cout << "\tMemory Partition (current): "
+                    << current_memory_partition << "\n\n";
+                } else {
+                    std::cout << "\tamdsmi_get_gpu_memory_partition(" << j << ", "
+                    << memoryPartitionString(AMDSMI_MEMORY_PARTITION_UNKNOWN) << "): "
+                    << err_str << "\n\n";
+                }
+            }
+            // return to original compute partition
+            amdsmi_memory_partition_type_t original_memory_partition_type;
+            if (ret == AMDSMI_STATUS_SUCCESS) {
+                original_memory_partition_type
+                    = mapStringToSMIMemoryPartitionTypes.at(original_memory_partition);
+            } else {
+                original_memory_partition_type = AMDSMI_MEMORY_PARTITION_UNKNOWN;
+            }
+            std::cout << "    Returning to original memory partition ("
+                      << memoryPartitionString(original_memory_partition_type)
+                      << ")\n";
+            ret_set = amdsmi_set_gpu_memory_partition(processor_handles[j],
+                                                original_memory_partition_type);
+            amdsmi_status_code_to_string(ret_set, &err_str);
+            if (ret_set == AMDSMI_STATUS_SUCCESS) {
+                CHK_AMDSMI_RET(ret_set)
+            }
+            std::cout << "\tamdsmi_set_gpu_compute_partition(" << j << ", "
+                      << memoryPartitionString(original_memory_partition_type) << "): "
+                      << err_str << "\n\n";
+
+            // TODO(amdsmi_team): Add V2 partiton APIs
+
             // Get VRAM info
             amdsmi_vram_info_t vram_info = {};
             ret = amdsmi_get_gpu_vram_info(processor_handles[j], &vram_info);
@@ -313,9 +529,11 @@ int main() {
                 CHK_AMDSMI_RET(ret)
                 printf("    Output of amdsmi_get_gpu_vram_info:\n");
                 printf("\tVRAM Size: 0x%lx (%ld) \n", vram_info.vram_size, vram_info.vram_size);
-                printf("\tBIT Width: 0x%x (%d) \n\n", vram_info.vram_bit_width, vram_info.vram_bit_width);
-            }
-            else {
+                printf("\tBIT Width: 0x%x (%d) \n\n", vram_info.vram_bit_width,
+                        vram_info.vram_bit_width);
+                printf("\tVRAM max bandwidth: 0x%lx (%lu) \n\n", vram_info.vram_max_bandwidth,
+                        vram_info.vram_max_bandwidth);
+            } else {
                 printf("\t**amdsmi_get_gpu_vram_info() not supported on this system.\n");
             }
 
@@ -429,12 +647,12 @@ int main() {
             printf("\tPCIe max speed: %d\n", pcie_info.pcie_static.max_pcie_speed);
 
             // additional pcie related metrics
-            printf("\tPCIe bandwidth: %d\n", pcie_info.pcie_metric.pcie_bandwidth);
-            printf("\tPCIe replay count: %d\n", pcie_info.pcie_metric.pcie_replay_count);
-            printf("\tPCIe L0 recovery count: %d\n", pcie_info.pcie_metric.pcie_l0_to_recovery_count);
-            printf("\tPCIe rollover count: %d\n", pcie_info.pcie_metric.pcie_replay_roll_over_count);
-            printf("\tPCIe nak received count: %d\n", pcie_info.pcie_metric.pcie_nak_received_count);
-            printf("\tPCIe nak sent count: %d\n", pcie_info.pcie_metric.pcie_nak_sent_count);
+            printf("\tPCIe bandwidth: %u\n", pcie_info.pcie_metric.pcie_bandwidth);
+            printf("\tPCIe replay count: %" PRIu64 "\n", pcie_info.pcie_metric.pcie_replay_count);
+            printf("\tPCIe L0 recovery count: %" PRIu64 "\n", pcie_info.pcie_metric.pcie_l0_to_recovery_count);
+            printf("\tPCIe rollover count: %" PRIu64 "\n", pcie_info.pcie_metric.pcie_replay_roll_over_count);
+            printf("\tPCIe nak received count: %" PRIu64 "\n", pcie_info.pcie_metric.pcie_nak_received_count);
+            printf("\tPCIe nak sent count: %" PRIu64 "\n", pcie_info.pcie_metric.pcie_nak_sent_count);
 
             // Get VRAM temperature limit
             int64_t temperature = 0;
@@ -496,7 +714,7 @@ int main() {
                  block = (amdsmi_gpu_block_t)(block * 2)) {
                 ret = amdsmi_get_gpu_ras_block_features_enabled(processor_handles[j], block,
                                                       &state);
-                if (ret != AMDSMI_STATUS_API_FAILED) {
+                if (ret != AMDSMI_STATUS_API_FAILED && ret != AMDSMI_STATUS_NOT_SUPPORTED) {
                     CHK_AMDSMI_RET(ret)
                 }
 
@@ -538,24 +756,19 @@ int main() {
             // Get ECC error counts
             amdsmi_error_count_t err_cnt_info = {};
             ret = amdsmi_get_gpu_total_ecc_count(processor_handles[j], &err_cnt_info);
-            CHK_AMDSMI_RET(ret)
+            if (ret != AMDSMI_STATUS_NOT_SUPPORTED) {
+                CHK_AMDSMI_RET(ret)
+            }
             printf("    Output of amdsmi_get_gpu_total_ecc_count:\n");
             printf("\tCorrectable errors: %lu\n", err_cnt_info.correctable_count);
             printf("\tUncorrectable errors: %lu\n\n",
                    err_cnt_info.uncorrectable_count);
-            // Get process list
-            auto compare = [](const void *a, const void *b) -> int {
-                return (*(amdsmi_proc_info_t *)a).pid >
-                               (*(amdsmi_proc_info_t *)b).pid
-                           ? 1
-                           : -1;
-            };
 
             uint32_t num_process = 0;
             ret = amdsmi_get_gpu_process_list(processor_handles[j], &num_process, nullptr);
             CHK_AMDSMI_RET(ret)
             if (!num_process) {
-                printf("No processes found.\n");
+                printf("amdsmi_get_gpu_process_list(): No processes found.\n\n");
             } else {
                 std::cout << "Processes found: " << num_process << "\n";
                 amdsmi_proc_info_t process_info_list[num_process];
@@ -563,18 +776,17 @@ int main() {
                 uint64_t mem = 0, gtt_mem = 0, cpu_mem = 0, vram_mem = 0;
                 uint64_t gfx = 0, enc = 0;
                 char bdf_str[20];
-                sprintf(bdf_str, "%04lx:%02x:%02x.%d",
-                        bdf.domain_number,
-                        bdf.bus_number,
-                        bdf.device_number,
-                        bdf.function_number);
-                int num = 0;
+                sprintf(bdf_str, "%04" PRIx64 ":%02" PRIx32 ":%02" PRIx32 ".%" PRIu32,
+                   static_cast<uint64_t>(bdf.domain_number),
+                   static_cast<uint32_t>(bdf.bus_number),
+                   static_cast<uint32_t>(bdf.device_number),
+                   static_cast<uint32_t>(bdf.function_number));
                 ret = amdsmi_get_gpu_process_list(processor_handles[j], &num_process, process_info_list);
                 std::cout << "Allocation size for process list: " << num_process << "\n";
                 CHK_AMDSMI_RET(ret);
                 for (auto idx = uint32_t(0); idx < num_process; ++idx) {
                     process = static_cast<amdsmi_proc_info_t>(process_info_list[idx]);
-                    printf("\t *Process id: %ld / Name: %s / VRAM: %lld \n", process.pid, process.name, process.memory_usage.vram_mem);
+                    printf("\t *Process id: %d / Name: %s / VRAM: %ld \n", process.pid, process.name, process.memory_usage.vram_mem);
                 }
 
                 printf("+=======+==================+============+=============="
@@ -590,7 +802,7 @@ int main() {
                 printf("+=======+"
                        "+=============+=============+=============+============"
                        "==+=========================================+\n");
-                for (int it = 0; it < num_process; it++) {
+                for (int it = 0; it < static_cast<int>(num_process); it++) {
                     char command[30];
                     struct passwd *pwd = nullptr;
                     struct stat st;
@@ -630,11 +842,13 @@ int main() {
                         "-+-------------+-------------+-------------+----------"
                         "----+-----------------------------------------+\n");
                 }
+                // TODO: To remove compiler warning, the last 3 values in this printf were
+                //       set to 0L.  Need to find out what these values need to be.
                 printf("|                                 TOTAL:| %s | %7ld "
                        "KiB | %7ld KiB | %7ld KiB | %7ld KiB | %lu  %lu  "
                        "%lu  %lu  %lu   |\n",
                        bdf_str, mem, gtt_mem, cpu_mem, vram_mem, gfx,
-                       enc);
+                       enc, 0L, 0L, 0L);
                 printf("+=======+==================+============+=============="
                        "+=============+=============+=============+============"
                        "=+==========================================+\n");
@@ -695,11 +909,11 @@ int main() {
             ret = amdsmi_get_gpu_metrics_info(processor_handles[j], &smu);
             CHK_AMDSMI_RET(ret)
             printf("    Output of amdsmi_get_gpu_metrics_info:\n");
-            printf("\tDevice[%d] BDF %04lx:%02x:%02x.%d\n\n", i,
-                   bdf.domain_number,
-                   bdf.bus_number,
-                   bdf.device_number,
-                   bdf.function_number);
+            printf("\tDevice[%d] BDF %04" PRIx64 ":%02" PRIx32 ":%02" PRIx32 ".%" PRIu32 "\n\n", i,
+                   static_cast<uint64_t>(bdf.domain_number),
+                   static_cast<uint32_t>(bdf.bus_number),
+                   static_cast<uint32_t>(bdf.device_number),
+                   static_cast<uint32_t>(bdf.function_number));
 
             std::cout << "METRIC TABLE HEADER:\n";
             std::cout << "structure_size=" << std::dec
@@ -727,7 +941,7 @@ int main() {
             auto idx = 0;
             for (const auto& temp : smu.temperature_hbm) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.temperature_hbm)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.temperature_hbm))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -744,7 +958,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.vcn_activity) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.vcn_activity)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.vcn_activity))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -757,7 +971,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.jpeg_activity) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.jpeg_activity)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.jpeg_activity))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -794,7 +1008,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.current_gfxclks) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.current_gfxclks)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.current_gfxclks))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -807,7 +1021,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.current_socclks) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.current_socclks)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.current_socclks))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -821,7 +1035,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.current_vclk0s) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.current_vclk0s)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.current_vclk0s))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -834,7 +1048,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.current_dclk0s) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.current_dclk0s)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.current_dclk0s))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -871,7 +1085,7 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.xgmi_read_data_acc) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.xgmi_read_data_acc)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.xgmi_read_data_acc))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -883,7 +1097,19 @@ int main() {
             idx = 0;
             for (const auto& temp : smu.xgmi_write_data_acc) {
               std::cout << temp;
-              if ((idx + 1) != std::size(smu.xgmi_write_data_acc)) {
+              if ((idx + 1) != static_cast<int>(std::size(smu.xgmi_write_data_acc))) {
+                std::cout << ", ";
+              } else {
+                std::cout << "]\n";
+              }
+              ++idx;
+            }
+
+            std::cout << std::dec << "\txgmi_link_status= [";
+            idx = 0;
+            for (const auto& temp : smu.xgmi_link_status) {
+              std::cout << temp;
+              if ((idx + 1) != static_cast<int>(std::size(smu.xgmi_link_status))) {
                 std::cout << ", ";
               } else {
                 std::cout << "]\n";
@@ -906,6 +1132,9 @@ int main() {
             std::cout << "\tpcie_bandwidth_acc=" << std::dec << smu.pcie_bandwidth_acc << "\n";
             std::cout << "\tpcie_bandwidth_inst=" << std::dec << smu.pcie_bandwidth_inst << "\n";
 
+            // VRAM max bandwidth at max memory clock
+            std::cout << "\tvram_max_bandwidth=" << std::dec << smu.vram_max_bandwidth << "\n";
+
             // Counts
             std::cout << "\tpcie_l0_to_recov_count_acc= " << std::dec << smu.pcie_l0_to_recov_count_acc
                       << "\n";
@@ -943,7 +1172,7 @@ int main() {
             for (auto& row : smu.xcp_stats) {
               std::cout << "\t XCP [" << idx << "] : [";
               for (auto& col : row.gfx_busy_inst) {
-                if ((idy + 1) != std::size(row.gfx_busy_inst)) {
+                if ((idy + 1) != static_cast<int>(std::size(row.gfx_busy_inst))) {
                     std::cout << col << ", ";
                 } else {
                     std::cout << col;
@@ -961,7 +1190,7 @@ int main() {
             for (auto& row : smu.xcp_stats) {
               std::cout << "\t XCP [" << idx << "] : [";
               for (auto& col : row.vcn_busy) {
-                if ((idy + 1) != std::size(row.vcn_busy)) {
+                if ((idy + 1) != static_cast<int>(std::size(row.vcn_busy))) {
                     std::cout << col << ", ";
                 } else {
                     std::cout << col;
@@ -979,7 +1208,7 @@ int main() {
             for (auto& row : smu.xcp_stats) {
               std::cout << "\t XCP [" << idx << "] : [";
               for (auto& col : row.jpeg_busy) {
-                if ((idy + 1) != std::size(row.jpeg_busy)) {
+                if ((idy + 1) != static_cast<int>(std::size(row.jpeg_busy))) {
                     std::cout << col << ", ";
                 } else {
                     std::cout << col;
@@ -997,7 +1226,25 @@ int main() {
             for (auto& row : smu.xcp_stats) {
               std::cout << "\t XCP [" << idx << "] : [";
               for (auto& col : row.gfx_busy_acc) {
-                if ((idy + 1) != std::size(row.gfx_busy_acc)) {
+                if ((idy + 1) != static_cast<int>(std::size(row.gfx_busy_acc))) {
+                    std::cout << col << ", ";
+                } else {
+                    std::cout << col;
+                }
+                idy++;
+              }
+              std::cout << "]\n";
+              idy = 0;
+              idx++;
+            }
+
+            idx = 0;
+            idy = 0;
+            std::cout  << "\txcp_stats.gfx_below_host_limit_acc: " << "\n";
+            for (auto& row : smu.xcp_stats) {
+              std::cout << "\t XCP [" << idx << "] : [";
+              for (auto& col : row.gfx_below_host_limit_acc) {
+                if ((idy + 1) != static_cast<int>(std::size(row.gfx_below_host_limit_acc))) {
                     std::cout << col << ", ";
                 } else {
                     std::cout << col;
@@ -1061,8 +1308,11 @@ int main() {
                     amdsmi_bdf_t bdf = {};
                     ret = amdsmi_get_gpu_device_bdf(topology_nearest_info.processor_list[k], &bdf);
                     CHK_AMDSMI_RET(ret)
-                    printf("\t\tGPU BDF %04lx:%02x:%02x.%d\n", bdf.domain_number,
-                        bdf.bus_number, bdf.device_number, bdf.function_number);
+                    printf("\t\tGPU BDF %04" PRIx64 ":%02" PRIx32 ":%02" PRIx32 ".%" PRIu32 "\n",
+                        static_cast<uint64_t>(bdf.domain_number),
+                        static_cast<uint32_t>(bdf.bus_number),
+                        static_cast<uint32_t>(bdf.device_number),
+                        static_cast<uint32_t>(bdf.function_number));
                 }
             }
       }
