@@ -1,48 +1,27 @@
 /*
- * =============================================================================
- * The University of Illinois/NCSA
- * Open Source License (NCSA)
- *
- * Copyright (c) 2024, Advanced Micro Devices, Inc.
- * All rights reserved.
- *
- * Developed by:
- *
- *                 AMD Research and AMD ROC Software Development
- *
- *                 Advanced Micro Devices, Inc.
- *
- *                 www.amd.com
+ * Copyright (c) Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to
- * deal with the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- *  - Redistributions of source code must retain the above copyright notice,
- *    this list of conditions and the following disclaimers.
- *  - Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimers in
- *    the documentation and/or other materials provided with the distribution.
- *  - Neither the names of <Name of Development Group, Name of Institution>,
- *    nor the names of its contributors may be used to endorse or promote
- *    products derived from this Software without specific prior written
- *    permission.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
- * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
- * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS WITH THE SOFTWARE.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
-
 #include <pwd.h>
+#include <inttypes.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -136,11 +115,11 @@ int main() {
             ret = amdsmi_get_gpu_device_bdf(processor_handles[j], &bdf);
             CHK_AMDSMI_RET(ret)
             printf("    Output of amdsmi_get_gpu_device_bdf:\n");
-            printf("\tDevice[%d] BDF %04lx:%02x:%02x.%d\n\n", i,
-                   bdf.domain_number,
-                   bdf.bus_number,
-                   bdf.device_number,
-                   bdf.function_number);
+            printf("\tDevice[%d] BDF %04" PRIx64 ":%02" PRIx32 ":%02" PRIx32 ".%" PRIu32 "\n\n", i,
+                   static_cast<uint64_t>(bdf.domain_number),
+                   static_cast<uint32_t>(bdf.bus_number),
+                   static_cast<uint32_t>(bdf.device_number),
+                   static_cast<uint32_t>(bdf.function_number));
 
             amdsmi_asic_info_t asic_info = {};
             ret = amdsmi_get_gpu_asic_info(processor_handles[j], &asic_info);
@@ -341,7 +320,7 @@ int main() {
                 CHK_AMDSMI_RET(ret)
                 std::cout << "\t amdsmi_get_soc_pstate total:" << policy.num_supported
                         <<" current:" << policy.current << "\n";
-                for (int x=0; x < policy.num_supported; x++) {
+                for (uint32_t x=0; x < policy.num_supported; x++) {
                     std::cout << x <<": (" << policy.policies[x].policy_id
                     <<"," << policy.policies[x].policy_description << ")\n";
                 }
@@ -371,8 +350,11 @@ int main() {
                     amdsmi_bdf_t bdf = {};
                     ret = amdsmi_get_gpu_device_bdf(topology_nearest_info.processor_list[k], &bdf);
                     CHK_AMDSMI_RET(ret)
-                    printf("\tGPU BDF %04lx:%02x:%02x.%d\n", bdf.domain_number,
-                        bdf.bus_number, bdf.device_number, bdf.function_number);
+                    printf("\tGPU BDF %04" PRIx64 ":%02" PRIx32 ":%02" PRIx32 ".%" PRIu32 "\n",
+                        static_cast<uint64_t>(bdf.domain_number),
+                        static_cast<uint32_t>(bdf.bus_number),
+                        static_cast<uint32_t>(bdf.device_number),
+                        static_cast<uint32_t>(bdf.function_number));
                 }
             }
         }
