@@ -3,15 +3,12 @@ import os
 
 setup(
     name="amdsmi",
-    version="25.1.0+unknown",
+    version="25.3.0+unknown",
     author="AMD",
     author_email="amd-smi.support@amd.com",
     description="AMDSMI Python LIB - AMD GPU Monitoring Library",
     url="https://github.com/ROCm/amdsmi",
     packages=find_packages(),
-    install_requires=[
-        "PyYAML>=3.0",
-    ],
     classifiers=[
         "Programming Language :: Python :: 3",
     ],
